@@ -1,22 +1,75 @@
 package CPANPLUS::Internals::Source;
 
 use strict;
-use CPANPLUS::inc;
+
 use CPANPLUS::Error;
 use CPANPLUS::Module;
 use CPANPLUS::Module::Fake;
 use CPANPLUS::Module::Author;
 use CPANPLUS::Internals::Constants;
 
+use File::Fetch;
 use Archive::Extract;
 
-use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
-use Params::Check               qw[check];
 use IPC::Cmd                    qw[can_run];
+use File::Temp                  qw[tempdir];
+use File::Basename              qw[dirname];
+use Params::Check               qw[check];
 use Module::Load::Conditional   qw[can_load];
+use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 
 $Params::Check::VERBOSE = 1;
 
+### list of methods the parent class must implement
+{   for my $sub ( qw[_init_trees _finalize_trees 
+                     _standard_trees_completed _custom_trees_completed
+                     _add_module_object _add_author_object _save_state
+                    ] 
+    ) {
+        no strict 'refs';
+        *$sub = sub { 
+            my $self    = shift;
+            my $class   = ref $self || $self;
+            
+            require Carp; 
+            Carp::croak( loc( "Class %1 must implement method '%2'", 
+                              $class, $sub ) );
+        }
+    }
+}    
+
+{
+    my $recurse; # flag to prevent recursive calls to *_tree functions
+
+    ### lazy loading of module tree
+    sub _module_tree {
+        my $self = $_[0];
+
+        unless ($self->_mtree or $recurse++ > 0) {
+            my $uptodate = $self->_check_trees( @_[1..$#_] );
+            $self->_build_trees(uptodate => $uptodate);
+        }
+
+        $recurse--;
+        return $self->_mtree;
+    }
+
+    ### lazy loading of author tree
+    sub _author_tree {
+        my $self = $_[0];
+
+        unless ($self->_atree or $recurse++ > 0) {
+            my $uptodate = $self->_check_trees( @_[1..$#_] );
+            $self->_build_trees(uptodate => $uptodate);
+        }
+
+        $recurse--;
+        return $self->_atree;
+    }
+
+}
+
+
 =pod
 
 =head1 NAME
@@ -42,17 +95,25 @@ well as update them, and then parse them.
 The flow looks like this:
 
     $cb->_author_tree || $cb->_module_tree
-        $cb->__check_trees
+        $cb->_check_trees
             $cb->__check_uptodate
                 $cb->_update_source
+            $cb->__update_custom_module_sources 
+                $cb->__update_custom_module_source
         $cb->_build_trees
+            ### engine methods
+            {   $cb->_init_trees;
+                $cb->_standard_trees_completed
+                $cb->_custom_trees_completed
+            }                
             $cb->__create_author_tree
-                $cb->__retrieve_source
+                ### engine methods
+                { $cb->_add_author_object }
             $cb->__create_module_tree
-                $cb->__retrieve_source
                 $cb->__create_dslip_tree
-                    $cb->__retrieve_source
-            $cb->_save_source
+                ### engine methods
+                { $cb->_add_module_object }
+            $cb->__create_custom_module_entries                    
 
     $cb->_dslip_defs
 
@@ -60,35 +121,127 @@ The flow looks like this:
 
 =cut
 
-{
-    my $recurse; # flag to prevent recursive calls to *_tree functions
+=pod
 
-    ### lazy loading of module tree
-    sub _module_tree {
-        my $self = $_[0];
+=head2 $cb->_build_trees( uptodate => BOOL, [use_stored => BOOL, path => $path, verbose => BOOL] )
 
-        unless ($self->{_modtree} or $recurse++ > 0) {
-            my $uptodate = $self->_check_trees( @_[1..$#_] );
-            $self->_build_trees(uptodate => $uptodate);
+This method rebuilds the author- and module-trees from source.
+
+It takes the following arguments:
+
+=over 4
+
+=item uptodate
+
+Indicates whether any on disk caches are still ok to use.
+
+=item path
+
+The absolute path to the directory holding the source files.
+
+=item verbose
+
+A boolean flag indicating whether or not to be verbose.
+
+=item use_stored
+
+A boolean flag indicating whether or not it is ok to use previously
+stored trees. Defaults to true.
+
+=back
+
+Returns a boolean indicating success.
+
+=cut
+
+### (re)build the trees ###
+sub _build_trees {
+    my ($self, %hash)   = @_;
+    my $conf            = $self->configure_object;
+
+    my($path,$uptodate,$use_stored,$verbose);
+    my $tmpl = {
+        path        => { default => $conf->get_conf('base'), store => \$path },
+        verbose     => { default => $conf->get_conf('verbose'), store => \$verbose },
+        uptodate    => { required => 1, store => \$uptodate },
+        use_stored  => { default => 1, store => \$use_stored },
+    };
+
+    my $args = check( $tmpl, \%hash ) or return;
+
+    $self->_init_trees(
+        path        => $path,
+        uptodate    => $uptodate,
+        verbose     => $verbose,
+        use_stored  => $use_stored,
+    ) or do {
+        error( loc("Could not initialize trees" ) );
+        return;
+    };        
+
+    ### return if we weren't able to build the trees ###
+    return unless $self->_mtree && $self->_atree;
+ 
+    ### did we get everything from a stored state? if not,
+    ### process them now.
+    if( not $self->_standard_trees_completed ) {
+     
+        ### first, prep the author tree
+        $self->__create_author_tree(
+                uptodate    => $uptodate,
+                path        => $path,
+                verbose     => $verbose, 
+        );
+
+        ### and now the module tree
+        $self->_create_mod_tree(
+                uptodate    => $uptodate,
+                path        => $path,
+                verbose     => $verbose, 
+        );
+    }
+    
+    ### XXX unpleasant hack. since custom sources uses ->parse_module, we
+    ### already have a special module object with extra meta data. that 
+    ### doesn't gelwell with the sqlite storage engine. So, we check 'normal'
+    ### trees from seperate trees, so the engine can treat them differently.
+    ### Effectively this means that with the SQLite engine, for now, custom
+    ### sources are continuously reparsed =/ -kane
+    if( not $self->_custom_trees_completed ) {
+    
+        ### update them if the other sources are also deemed out of date
+        if( $conf->get_conf('enable_custom_sources') ) {
+            $self->__update_custom_module_sources( verbose => $verbose ) 
+                or error(loc("Could not update custom module sources"));
+        }      
+
+        ### add custom sources here if enabled
+        if( $conf->get_conf('enable_custom_sources') ) {
+            $self->__create_custom_module_entries( verbose => $verbose )
+                or error(loc("Could not create custom module entries"));
         }
-
-        $recurse--;
-        return $self->{_modtree};
     }
 
-    ### lazy loading of author tree
-    sub _author_tree {
-        my $self = $_[0];
-
-        unless ($self->{_authortree} or $recurse++ > 0) {
-            my $uptodate = $self->_check_trees( @_[1..$#_] );
-            $self->_build_trees(uptodate => $uptodate);
-        }
-
-        $recurse--;
-        return $self->{_authortree};
-    }
+    ### give the source engine a chance to wrap up creation
+    $self->_finalize_trees(
+        path        => $path,
+        uptodate    => $uptodate,
+        verbose     => $verbose,    
+        use_stored  => $use_stored,
+    ) or do {
+        error(loc( "Could not finalize trees" ));
+        return;
+    };        
+    
+    ### still necessary? can only run one instance now ###
+    ### will probably stay that way --kane
+#     my $id = $self->_store_id( $self );
+#
+#     unless ( $id == $self->_id ) {
+#         error( loc("IDs do not match: %1 != %2. Storage failed!", $id, $self->_id) );
+#     }
 
+    return 1;
 }
 
 =pod
@@ -154,7 +307,7 @@ sub _check_trees {
     for my $name (qw[auth dslip mod]) {
         for my $file ( $conf->_get_source( $name ) ) {
             $self->__check_uptodate(
-                file            => File::Spec->catfile( $args->{path}, $file ),
+                file            => File::Spec->catfile( $path, $file ),
                 name            => $name,
                 update_source   => $update_source,
                 verbose         => $verbose,
@@ -162,6 +315,12 @@ sub _check_trees {
         }
     }
 
+    ### if we're explicitly asked to update the sources, or if the
+    ### standard source files are out of date, update the custom sources
+    ### as well
+    $self->__update_custom_module_sources( verbose => $verbose ) 
+        if $update_source or !$uptodate;
+
     return $uptodate;
 }
 
@@ -228,8 +387,8 @@ sub __check_uptodate {
     if ( $flag or $args->{'update_source'} ) {
 
          if ( $self->_update_source( name => $args->{'name'} ) ) {
-              return 0;       # return 0 so 'uptodate' will be set to 0, meaning no use
-                              # of previously stored hashrefs!
+              return 0;       # return 0 so 'uptodate' will be set to 0, meaning no 
+                              # use of previously stored hashrefs!
          } else {
               msg( loc("Unable to update source, attempting to get away with using old source file!"), $args->{verbose} );
               return 1;
@@ -275,25 +434,23 @@ sub _update_source {
     my %hash = @_;
     my $conf = $self->configure_object;
 
-
+    my $verbose;
     my $tmpl = {
         name    => { required => 1 },
         path    => { default => $conf->get_conf('base') },
-        verbose => { default => $conf->get_conf('verbose') },
+        verbose => { default => $conf->get_conf('verbose'), store => \$verbose },
     };
 
     my $args = check( $tmpl, \%hash ) or return;
 
 
     my $path = $args->{path};
-    my $now = time;
-
     {   ### this could use a clean up - Kane
         ### no worries about the / -> we get it from the _ftp configuration, so
         ### it's not platform dependant. -kane
         my ($dir, $file) = $conf->_get_mirror( $args->{'name'} ) =~ m|(.+/)(.+)$|sg;
 
-        msg( loc("Updating source file '%1'", $file), $args->{'verbose'} );
+        msg( loc("Updating source file '%1'", $file), $verbose );
 
         my $fake = CPANPLUS::Module::Fake->new(
                         module  => $args->{'name'},
@@ -316,264 +473,12 @@ sub _update_source {
             return;
         }
 
-        ### `touch` the file, so windoze knows it's new -jmb
-        ### works on *nix too, good fix -Kane
-        utime ( $now, $now, File::Spec->catfile($path, $file) ) or
-            error( loc("Couldn't touch %1", $file) );
-
+        $self->_update_timestamp( file => File::Spec->catfile($path, $file) );
     }
-    return 1;
-}
-
-=pod
-
-=head2 $cb->_build_trees( uptodate => BOOL, [use_stored => BOOL, path => $path, verbose => BOOL] )
-
-This method rebuilds the author- and module-trees from source.
-
-It takes the following arguments:
-
-=over 4
-
-=item uptodate
-
-Indicates whether any on disk caches are still ok to use.
-
-=item path
-
-The absolute path to the directory holding the source files.
-
-=item verbose
-
-A boolean flag indicating whether or not to be verbose.
-
-=item use_stored
-
-A boolean flag indicating whether or not it is ok to use previously
-stored trees. Defaults to true.
-
-=back
-
-Returns a boolean indicating success.
-
-=cut
-
-### (re)build the trees ###
-sub _build_trees {
-    my ($self, %hash)   = @_;
-    my $conf            = $self->configure_object;
-
-    my($path,$uptodate,$use_stored);
-    my $tmpl = {
-        path        => { default => $conf->get_conf('base'), store => \$path },
-        verbose     => { default => $conf->get_conf('verbose') },
-        uptodate    => { required => 1, store => \$uptodate },
-        use_stored  => { default => 1, store => \$use_stored },
-    };
-
-    my $args = check( $tmpl, \%hash ) or return undef;
-
-    ### retrieve the stored source files ###
-    my $stored      = $self->__retrieve_source(
-                            path        => $path,
-                            uptodate    => $uptodate && $use_stored,
-                            verbose     => $args->{'verbose'},
-                        ) || {};
-
-    ### build the trees ###
-    $self->{_authortree} =  $stored->{_authortree} ||
-                            $self->__create_author_tree(
-                                    uptodate    => $uptodate,
-                                    path        => $path,
-                                    verbose     => $args->{verbose},
-                                );
-    $self->{_modtree}    =  $stored->{_modtree} ||
-                            $self->_create_mod_tree(
-                                    uptodate    => $uptodate,
-                                    path        => $path,
-                                    verbose     => $args->{verbose},
-                                );
-
-    ### return if we weren't able to build the trees ###
-    return unless $self->{_modtree} && $self->{_authortree};
-
-    ### write the stored files to disk, so we can keep using them
-    ### from now on, till they become invalid
-    ### write them if the original sources weren't uptodate, or
-    ### we didn't just load storable files
-    $self->_save_source() if !$uptodate or not keys %$stored;
-
-    ### still necessary? can only run one instance now ###
-    ### will probably stay that way --kane
-#     my $id = $self->_store_id( $self );
-#
-#     unless ( $id == $self->_id ) {
-#         error( loc("IDs do not match: %1 != %2. Storage failed!", $id, $self->_id) );
-#     }
 
     return 1;
 }
 
-=pod
-
-=head2 $cb->__retrieve_source(name => $name, [path => $path, uptodate => BOOL, verbose => BOOL])
-
-This method retrieves a I<storable>d tree identified by C<$name>.
-
-It takes the following arguments:
-
-=over 4
-
-=item name
-
-The internal name for the source file to retrieve.
-
-=item uptodate
-
-A flag indicating whether the file-cache is up-to-date or not.
-
-=item path
-
-The absolute path to the directory holding the source files.
-
-=item verbose
-
-A boolean flag indicating whether or not to be verbose.
-
-=back
-
-Will get information from the config file by default.
-
-Returns a tree on success, false on failure.
-
-=cut
-
-sub __retrieve_source {
-    my $self = shift;
-    my %hash = @_;
-    my $conf = $self->configure_object;
-
-    my $tmpl = {
-        path     => { default => $conf->get_conf('base') },
-        verbose  => { default => $conf->get_conf('verbose') },
-        uptodate => { default => 0 },
-    };
-
-    my $args = check( $tmpl, \%hash ) or return;
-
-    ### check if we can retrieve a frozen data structure with storable ###
-    my $storable = can_load( modules => {'Storable' => '0.0'} )
-                        if $conf->get_conf('storable');
-
-    return unless $storable;
-
-    ### $stored is the name of the frozen data structure ###
-    my $stored = File::Spec->rel2abs(
-                    File::Spec->catfile(
-                                $args->{path},      #base dir
-                                #$args->{name}       #file
-                                $conf->_get_source('stored')
-                                . '.' .
-                                $Storable::VERSION  #the version of storable used to store
-                                . '.stored'         #append a suffix
-                    )
-                );
-
-    if ($storable && -e $stored && -s _ && $args->{'uptodate'}) {
-        msg( loc("Retrieving %1", $stored), $args->{'verbose'} );
-
-        my $href = Storable::retrieve($stored);
-        return $href;
-    } else {
-        return;
-    }
-}
-
-=pod
-
-=head2 $cb->_save_source([verbose => BOOL, path => $path])
-
-This method saves all the parsed trees in I<storable>d format if
-C<Storable> is available.
-
-It takes the following arguments:
-
-=over 4
-
-=item path
-
-The absolute path to the directory holding the source files.
-
-=item verbose
-
-A boolean flag indicating whether or not to be verbose.
-
-=back
-
-Will get information from the config file by default.
-
-Returns true on success, false on failure.
-
-=cut
-
-sub _save_source {
-    my $self = shift;
-    my %hash = @_;
-    my $conf = $self->configure_object;
-
-
-    my $tmpl = {
-        path     => { default => $conf->get_conf('base'), allow => DIR_EXISTS },
-        verbose  => { default => $conf->get_conf('verbose') },
-        force    => { default => 1 },
-    };
-
-    my $args = check( $tmpl, \%hash ) or return;
-
-    my $aref = [qw[_modtree _authortree]];
-
-    ### check if we can retrieve a frozen data structure with storable ###
-    my $storable;
-    $storable = can_load( modules => {'Storable' => '0.0'} )
-                    if $conf->get_conf('storable');
-    return unless $storable;
-
-    my $to_write = {};
-    foreach my $key ( @$aref ) {
-        next unless ref( $self->{$key} );
-        $to_write->{$key} = $self->{$key};
-    }
-
-    return unless keys %$to_write;
-
-    ### $stored is the name of the frozen data structure ###
-    ### changed to use File::Spec->catfile -jmb
-    my $stored = File::Spec->catfile(
-                            $args->{path},                  #base dir
-                            $conf->_get_source('stored')    #file
-                            . '.' .
-                            $Storable::VERSION              #the version of storable used to store
-                            . '.stored'                     #append a suffix
-                        );
-
-    if (-e $stored && not -w $stored) {
-        msg( loc("%1 not writable; skipped.", $stored), $args->{'verbose'} );
-        return;
-    }
-
-    msg( loc("Writing compiled source information to disk. This might take a little while."),
-	    $args->{'verbose'} );
-
-    my $flag;
-    unless( Storable::nstore( $to_write, $stored ) ) {
-        error( loc("could not store %1!", $stored) );
-        $flag++;
-    }
-
-    return $flag ? 0 : 1;
-}
-
-
 =pod
 
 =head2 $cb->__create_author_tree([path => $path, uptodate => BOOL, verbose => BOOL])
@@ -606,7 +511,7 @@ Returns a tree on success, false on failure.
 
 =cut
 
-sub __create_author_tree() {
+sub __create_author_tree {
     my $self = shift;
     my %hash = @_;
     my $conf = $self->configure_object;
@@ -619,7 +524,7 @@ sub __create_author_tree() {
     };
 
     my $args = check( $tmpl, \%hash ) or return;
-    my $tree = {};
+
     my $file = File::Spec->catfile(
                                 $args->{path},
                                 $conf->_get_source('auth')
@@ -648,15 +553,15 @@ sub __create_author_tree() {
                                     "\s* ([^\"\<]+?) \s* <(.+)> \s*"
                                 /x;
 
-        $tree->{$id} = CPANPLUS::Module::Author->new(
+        $self->_add_author_object(
             author  => $name,           #authors name
             email   => $email,          #authors email address
             cpanid  => $id,             #authors CPAN ID
-            _id     => $self->_id,    #id of this internals object
-        );
+        ) or error( loc("Could not add author '%1'", $name ) );
+
     }
 
-    return $tree;
+    return $self->_atree;
 
 } #__create_author_tree
 
@@ -728,7 +633,6 @@ sub _create_mod_tree {
     ### don't need it anymore ###
     unlink $out;
 
-    my $tree = {};
     my $flag;
 
     for ( split /\n/, $cont ) {
@@ -748,8 +652,8 @@ sub _create_mod_tree {
         ### authors can apparently have digits in their names,
         ### and dirs can have dots... blah!
         my ($author, $package) = $data[2] =~
-                m|  [A-Z\d-]/
-                    [A-Z\d-]{2}/
+                m|  (?:[A-Z\d-]/)?
+                    (?:[A-Z\d-]{2}/)?
                     ([A-Z\d-]+) (?:/[\S]+)?/
                     ([^/]+)$
                 |xsg;
@@ -757,8 +661,8 @@ sub _create_mod_tree {
         ### remove file name from the path
         $data[2] =~ s|/[^/]+$||;
 
-
-        unless( $self->author_tree($author) ) {
+        my $aobj = $self->author_tree($author);
+        unless( $aobj ) {
             error( loc( "No such author '%1' -- can't make module object " .
                         "'%2' that is supposed to belong to this author",
                         $author, $data[0] ) );
@@ -775,30 +679,35 @@ sub _create_mod_tree {
                             ? $dslip_tree->{ $data[0] }->{$item}
                             : ' ';
         }
+        
+        ### XXX this could be sped up if we used author names, not author
+        ### objects in creation, and then look them up in the author tree
+        ### when needed. This will need a fix to all the places that create
+        ### fake author/module objects as well.
 
-        ### Every module get's stored as a module object ###
-        $tree->{ $data[0] } = CPANPLUS::Module->new(
-                module      => $data[0],            # full module name
-                version     => ($data[1] eq 'undef' # version number 
-                                    ? '0.0' 
-                                    : $data[1]), 
-                path        => File::Spec::Unix->catfile(
-                                    $conf->_get_mirror('base'),
-                                    $data[2],
-                                ),          # extended path on the cpan mirror,
-                                            # like /A/AB/ABIGAIL
-                comment     => $data[3],    # comment on the module
-                author      => $self->author_tree($author),
-                package     => $package,    # package name, like
-                                            # 'foo-bar-baz-1.03.tar.gz'
-                description => $dslip_tree->{ $data[0] }->{'description'},
-                dslip       => $dslip,
-                _id         => $self->_id,  #id of this internals object
-        );
+        ### callback to store the individual object
+        $self->_add_module_object(
+            module      => $data[0],            # full module name
+            version     => ($data[1] eq 'undef' # version number 
+                                ? '0.0' 
+                                : $data[1]), 
+            path        => File::Spec::Unix->catfile(
+                                $conf->_get_mirror('base'),
+                                $data[2],
+                            ),          # extended path on the cpan mirror,
+                                        # like /A/AB/ABIGAIL
+            comment     => $data[3],    # comment on the module
+            author      => $aobj,
+            package     => $package,    # package name, like
+                                        # 'foo-bar-baz-1.03.tar.gz'
+            description => $dslip_tree->{ $data[0] }->{'description'},
+            dslip       => $dslip,
+            mtime       => '',
+        ) or error( loc( "Could not add module '%1'", $data[0] ) );
 
     } #for
 
-    return $tree;
+    return $self->_mtree;
 
 } #_create_mod_tree
 
@@ -991,11 +900,493 @@ sub _dslip_defs {
     return $aref;
 }
 
-# Local variables:
-# c-indentation-style: bsd
-# c-basic-offset: 4
-# indent-tabs-mode: nil
-# End:
-# vim: expandtab shiftwidth=4:
+=head2 $file = $cb->_add_custom_module_source( uri => URI, [verbose => BOOL] ); 
+
+Adds a custom source index and updates it based on the provided URI.
+
+Returns the full path to the index file on success or false on failure.
+
+=cut
+
+sub _add_custom_module_source {
+    my $self = shift;
+    my $conf = $self->configure_object;
+    my %hash = @_;
+    
+    my($verbose,$uri);
+    my $tmpl = {   
+        verbose => { default => $conf->get_conf('verbose'),
+                     store   => \$verbose },
+        uri     => { required => 1, store => \$uri }
+    };
+    
+    check( $tmpl, \%hash ) or return;
+    
+    ### what index file should we use on disk?
+    my $index = $self->__custom_module_source_index_file( uri => $uri );
+
+    ### already have it.
+    if( IS_FILE->( $index ) ) {
+        msg(loc("Source '%1' already added", $uri));
+        return 1;
+    }        
+        
+    ### do we need to create the targe dir?        
+    {   my $dir = dirname( $index );
+        unless( IS_DIR->( $dir ) ) {
+            $self->_mkdir( dir => $dir ) or return
+        }
+    }  
+    
+    ### write the file
+    my $fh = OPEN_FILE->( $index => '>' ) or do {
+        error(loc("Could not open index file for '%1'", $uri));
+        return;
+    };
+    
+    ### basically we 'touched' it. Check the return value, may be 
+    ### important on win32 and similar OS, where there's file length
+    ### limits
+    close $fh or do {
+        error(loc("Could not write index file to disk for '%1'", $uri));
+        return;
+    };        
+        
+    $self->__update_custom_module_source(
+                remote  => $uri,
+                local   => $index,
+                verbose => $verbose,
+            ) or do {
+                ### we faild to update it, we probably have an empty
+                ### possibly silly filename on disk now -- remove it
+                1 while unlink $index;
+                return;                
+            };
+            
+    return $index;
+}
+
+=head2 $index = $cb->__custom_module_source_index_file( uri => $uri );
+
+Returns the full path to the encoded index file for C<$uri>, as used by
+all C<custom module source> routines.
+
+=cut
+
+sub __custom_module_source_index_file {
+    my $self = shift;
+    my $conf = $self->configure_object;
+    my %hash = @_;
+    
+    my($verbose,$uri);
+    my $tmpl = {   
+        uri     => { required => 1, store => \$uri }
+    };
+    
+    check( $tmpl, \%hash ) or return;
+    
+    my $index = File::Spec->catfile(
+                    $conf->get_conf('base'),
+                    $conf->_get_build('custom_sources'),        
+                    $self->_uri_encode( uri => $uri ),
+                );     
+
+    return $index;
+}
+
+=head2 $file = $cb->_remove_custom_module_source( uri => URI, [verbose => BOOL] ); 
+
+Removes a custom index file based on the URI provided.
+
+Returns the full path to the index file on success or false on failure.
+
+=cut
+
+sub _remove_custom_module_source {
+    my $self = shift;
+    my $conf = $self->configure_object;
+    my %hash = @_;
+    
+    my($verbose,$uri);
+    my $tmpl = {   
+        verbose => { default => $conf->get_conf('verbose'),
+                     store   => \$verbose },
+        uri     => { required => 1, store => \$uri }
+    };
+    
+    check( $tmpl, \%hash ) or return;
+
+    ### use uri => local, instead of the other way around
+    my %files = reverse $self->__list_custom_module_sources;
+    
+    ### On VMS the case of key to %files can be either exact or lower case
+    ### XXX abstract this lookup out? --kane
+    my $file = $files{ $uri };
+    $file    = $files{ lc $uri } if !defined($file) && ON_VMS;
+
+    unless (defined $file) {
+        error(loc("No such custom source '%1'", $uri));
+        return;
+    };
+                
+    1 while unlink $file;
+ 
+    if( IS_FILE->( $file ) ) {
+        error(loc("Could not remove index file '%1' for custom source '%2'",
+                    $file, $uri));
+        return;
+    }    
+            
+    msg(loc("Successfully removed index file for '%1'", $uri), $verbose);
+
+    return $file;
+}
+
+=head2 %files = $cb->__list_custom_module_sources
+
+This method scans the 'custom-sources' directory in your base directory
+for additional sources to include in your module tree.
+
+Returns a list of key value pairs as follows:
+
+  /full/path/to/source/file%3Fencoded => http://decoded/mirror/path
+
+=cut
+
+sub __list_custom_module_sources {
+    my $self = shift;
+    my $conf = $self->configure_object;
+    
+    my($verbose);
+    my $tmpl = {   
+        verbose => { default => $conf->get_conf('verbose'),
+                     store   => \$verbose },
+    };    
+
+    my $dir = File::Spec->catdir(
+                    $conf->get_conf('base'),
+                    $conf->_get_build('custom_sources'),
+                );
+
+    unless( IS_DIR->( $dir ) ) {
+        msg(loc("No '%1' dir, skipping custom sources", $dir), $verbose);
+        return;
+    }
+    
+    ### unencode the files
+    ### skip ones starting with # though
+    my %files = map {            
+        my $org = $_;            
+        my $dec = $self->_uri_decode( uri => $_ );            
+        File::Spec->catfile( $dir, $org ) => $dec
+    } grep { $_ !~ /^#/ } READ_DIR->( $dir );        
+
+    return %files;    
+}
+
+=head2 $bool = $cb->__update_custom_module_sources( [verbose => BOOL] );
+
+Attempts to update all the index files to your custom module sources.
+
+If the index is missing, and it's a C<file://> uri, it will generate
+a new local index for you.
+
+Return true on success, false on failure.
+
+=cut
+
+sub __update_custom_module_sources {
+    my $self = shift;
+    my $conf = $self->configure_object;
+    my %hash = @_;
+    
+    my $verbose;
+    my $tmpl = {   
+        verbose => { default => $conf->get_conf('verbose'),
+                     store   => \$verbose }
+    };
+    
+    check( $tmpl, \%hash ) or return;
+    
+    my %files = $self->__list_custom_module_sources;
+    
+    ### uptodate check has been done a few levels up.   
+    my $fail;
+    while( my($local,$remote) = each %files ) {
+        
+        $self->__update_custom_module_source(
+                    remote  => $remote,
+                    local   => $local,
+                    verbose => $verbose,
+                ) or ( $fail++, next );         
+    }
+    
+    error(loc("Failed updating one or more remote sources files")) if $fail;
+    
+    return if $fail;
+    return 1;
+}
+
+=head2 $ok = $cb->__update_custom_module_source 
+
+Attempts to update all the index files to your custom module sources.
+
+If the index is missing, and it's a C<file://> uri, it will generate
+a new local index for you.
+
+Return true on success, false on failure.
+
+=cut
+
+sub __update_custom_module_source {
+    my $self = shift;
+    my $conf = $self->configure_object;
+    my %hash = @_;
+    
+    my($verbose,$local,$remote);
+    my $tmpl = {   
+        verbose => { default  => $conf->get_conf('verbose'),
+                     store    => \$verbose },
+        local   => { store    => \$local, allow => FILE_EXISTS },
+        remote  => { required => 1, store => \$remote },
+    };
+
+    check( $tmpl, \%hash ) or return;
+
+    msg( loc("Updating sources from '%1'", $remote), $verbose);
+    
+    ### if you didn't provide a local file, we'll look in your custom
+    ### dir to find the local encoded version for you
+    $local ||= do {
+        ### find all files we know of
+        my %files = reverse $self->__list_custom_module_sources or do {
+            error(loc("No custom modules sources defined -- need '%1' argument",
+                      'local'));
+            return;                      
+        };
+
+        ### On VMS the case of key to %files can be either exact or lower case
+        ### XXX abstract this lookup out? --kane
+        my $file = $files{ $remote };
+        $file    = $files{ lc $remote } if !defined ($file) && ON_VMS;
+
+        ### return the local file we're supposed to use
+        $file or do {
+            error(loc("Remote source '%1' unknown -- needs '%2' argument",
+                      $remote, 'local'));
+            return;
+        };         
+    };
+    
+    my $uri =  join '/', $remote, $conf->_get_source('custom_index');
+    my $ff  =  File::Fetch->new( uri => $uri );           
+
+    ### tempdir doesn't clean up by default, as opposed to tempfile()
+    ### so add it explicitly.
+    my $dir =  tempdir( CLEANUP => 1 );
+    
+    my $res =  do {  local $File::Fetch::WARN = 0;
+                    local $File::Fetch::WARN = 0;
+                    $ff->fetch( to => $dir );
+                };
+
+    ### couldn't get the file
+    unless( $res ) {
+        
+        ### it's not a local scheme, so can't auto index
+        unless( $ff->scheme eq 'file' ) {
+            error(loc("Could not update sources from '%1': %2",
+                      $remote, $ff->error ));
+            return;   
+                        
+        ### it's a local uri, we can index it ourselves
+        } else {
+            msg(loc("No index file found at '%1', generating one",
+                    $ff->uri), $verbose );
+            
+            ### ON VMS, if you are working with a UNIX file specification,
+            ### you need currently use the UNIX variants of the File::Spec.
+            my $ff_path = do {
+                my $file_class = 'File::Spec';
+                $file_class .= '::Unix' if ON_VMS;
+                $file_class->catdir( File::Spec::Unix->splitdir( $ff->path ) );
+            };      
+
+            $self->__write_custom_module_index(
+                path    => $ff_path,
+                to      => $local,
+                verbose => $verbose,
+            ) or return;
+            
+            ### XXX don't write that here, __write_custom_module_index
+            ### already prints this out
+            #msg(loc("Index file written to '%1'", $to), $verbose);
+        }
+    
+    ### copy it to the real spot and update its timestamp
+    } else {            
+        $self->_move( file => $res, to => $local ) or return;
+        $self->_update_timestamp( file => $local );
+        
+        msg(loc("Index file saved to '%1'", $local), $verbose);
+    }
+    
+    return $local;
+}
+
+=head2 $bool = $cb->__write_custom_module_index( path => /path/to/packages, [to => /path/to/index/file, verbose => BOOL] )
+
+Scans the C<path> you provided for packages and writes an index with all 
+the available packages to C<$path/packages.txt>. If you'd like the index
+to be written to a different file, provide the C<to> argument.
+
+Returns true on success and false on failure.
+
+=cut
+
+sub __write_custom_module_index {
+    my $self = shift;
+    my $conf = $self->configure_object;
+    my %hash = @_;
+    
+    my ($verbose, $path, $to);
+    my $tmpl = {   
+        verbose => { default => $conf->get_conf('verbose'),
+                     store   => \$verbose },
+        path    => { required => 1, allow => DIR_EXISTS, store => \$path },
+        to      => { store => \$to },
+    };
+    
+    check( $tmpl, \%hash ) or return;    
+
+    ### no explicit to? then we'll use our default
+    $to ||= File::Spec->catfile( $path, $conf->_get_source('custom_index') );
+
+    my @files;
+    require File::Find;
+    File::Find::find( sub { 
+        ### let's see if A::E can even parse it
+        my $ae = do {
+            local $Archive::Extract::WARN = 0;
+            local $Archive::Extract::WARN = 0;
+            Archive::Extract->new( archive => $File::Find::name ) 
+        } or return; 
+
+        ### it's a type A::E recognize, so we can add it
+        $ae->type or return;
+
+        ### neither $_ nor $File::Find::name have the chunk of the path in
+        ### it starting $path -- it's either only the filename, or the full
+        ### path, so we have to strip it ourselves
+        ### make sure to remove the leading slash as well.
+        my $copy = $File::Find::name;
+        my $re   = quotemeta($path);        
+        $copy    =~ s|^$re[\\/]?||i;
+        
+        push @files, $copy;
+        
+    }, $path );
+
+    ### does the dir exist? if not, create it.
+    {   my $dir = dirname( $to );
+        unless( IS_DIR->( $dir ) ) {
+            $self->_mkdir( dir => $dir ) or return
+        }
+    }        
+
+    ### create the index file
+    my $fh = OPEN_FILE->( $to => '>' ) or return;
+    
+    print $fh "$_\n" for @files;
+    close $fh;
+    
+    msg(loc("Successfully written index file to '%1'", $to), $verbose);
+    
+    return $to;
+}
+
+
+=head2 $bool = $cb->__create_custom_module_entries( [verbose => BOOL] ) 
+
+Creates entries in the module tree based upon the files as returned
+by C<__list_custom_module_sources>.
+
+Returns true on success, false on failure.
+
+=cut 
+
+### use $auth_obj as a persistant version, so we don't have to recreate
+### modules all the time
+{   my $auth_obj; 
+
+    sub __create_custom_module_entries {
+        my $self    = shift;
+        my $conf    = $self->configure_object;
+        my %hash    = @_;
+        
+        my $verbose;
+        my $tmpl = {
+            verbose     => { default => $conf->get_conf('verbose'), store => \$verbose },
+        };
+    
+        check( $tmpl, \%hash ) or return undef;
+        
+        my %files = $self->__list_custom_module_sources;     
+    
+        while( my($file,$name) = each %files ) {
+            
+            msg(loc("Adding packages from custom source '%1'", $name), $verbose);
+    
+            my $fh = OPEN_FILE->( $file ) or next;
+    
+            while( local $_ = <$fh> ) {
+                chomp;
+                next if /^#/;
+                next unless /\S+/;
+                
+                ### join on / -- it's a URI after all!
+                my $parse = join '/', $name, $_;
+    
+                ### try to make a module object out of it
+                my $mod = $self->parse_module( module => $parse ) or (
+                    error(loc("Could not parse '%1'", $_)),
+                    next
+                );
+                
+                ### mark this object with a custom author
+                $auth_obj ||= do {
+                    my $id = CUSTOM_AUTHOR_ID;
+                    
+                    ### if the object is being created for the first time,
+                    ### make sure there's an entry in the author tree as
+                    ### well, so we can search on the CPAN ID
+                    $self->author_tree->{ $id } = 
+                        CPANPLUS::Module::Author::Fake->new( cpanid => $id );          
+                };
+                
+                $mod->author( $auth_obj );
+                
+                ### and now add it to the modlue tree -- this MAY
+                ### override things of course
+                if( my $old_mod = $self->module_tree( $mod->module ) ) {
+
+                    ### On VMS use the old module name to get the real case
+                    $mod->module( $old_mod->module ) if ON_VMS;
+
+                    msg(loc("About to overwrite module tree entry for '%1' with '%2'",
+                            $mod->module, $mod->package), $verbose);
+                }
+                
+                ### mark where it came from
+                $mod->description( loc("Custom source from '%1'",$name) );
+                
+                ### store it in the module tree
+                $self->module_tree->{ $mod->module } = $mod;
+            }
+        }
+        
+        return 1;
+    }
+}
 
 1;
